/* Binary I/O test */

#include "BinaryTest.h"

#include "bfstream.h"		// the new file stream

#include <iostream>

using namespace std;

////////

//	This tests outputs two files in binary mode using the standard stream (old.txt)
//	and the bofstream.(new.txt)
//	Use file compare (fc /b) to compare the two files

void Output_Binary()
{
	int index;			// looping var

	cout << "Binary output" << endl;
	cout << "--------------" << endl;

	ofstream	ofs("old.txt", ios::binary);		// a normal ofstream
	bofstream	bos("new.txt", ios::binary);		// the wrapper file stream in binary mode

	if (!ofs || !bos)
	{
		cout << "*******************************************************" << endl;
		cout << "*****************Error creating file*******************" << endl;
		cout << "*******************************************************" << endl;
	}

	// basic type output
	//------------------

	int		i	= 987;
	float	f	= -12.34f;
	double	d	= 87654321.141231234;

	ofs.write((const char *)&i, sizeof(i));		// old way
	ofs.write((const char *)&f, sizeof(f));
	ofs.write((const char *)&d, sizeof(d));

	bos << i << f << d;							// write basic types like this

	cout << i << " " << f << " " << d << endl;

	// basic array output
	//--------------------

	float f_a[5];

	for (index = 0; index < 5; ++index)
		f_a[index]	= -123.23f - index;

	ofs.write((const char *)f_a, sizeof(float)*5);	// old way

	bos.write(f_a, sizeof(float)*5);				// write arrays like this (no cast needed) but appears as garbage in text mode

	for (index = 0; index < 5; ++index)
		cout << f_a[index] << " ";
	cout << endl;

	// basic pointer output
	//----------------------

	float *f_p = new float[5];

	for (index = 0; index < 5; ++index)
		f_p[index] = 98.32f - index;

	ofs.write((const char *)f_p, sizeof(float)*5);		// old way

	for (index = 0; index < 5; ++index)					// slower to write pointer arrays like this
		bos << f_p[index];								// but values are viewable in text mode

	for (index = 0; index < 5; ++index)
		cout << f_p[index] << " ";
	cout << endl;

	delete	[]f_p;
	f_p	= NULL;

	// string output - char buffer
	//-----------------------------
	char buffer[20]	= "Char buffer";			// a char buffer

	ofs << buffer << ends;

	bos	<< buffer << ends;						// write a string(term by a '\0')

	cout << buffer << endl;

	// string output - raw char pointer
	//---------------------------------

	char *c_p	= new char[20];

	sprintf(c_p,"Char pointer");

	ofs << c_p << ends;

	bos << c_p << ends;							// write a string(term by a '\0')

	cout << c_p << endl;

	delete	[]c_p;
	c_p = NULL;

	// constant string
	//-----------------

	ofs		<< "Const string" << ends;

	bos		<< "Const string" << ends;			// write a const string(term by a '\0')

	cout	<< "Const string" << endl;

	// const string pointers
	//----------------------

	const char* const_p = "Const char string";

	ofs		<< const_p << ends;
	bos		<< const_p << ends;					// write a const string pointer(term by a '\0')

	cout	<< const_p << endl;

	const char* const cc2_p = "const char const string";

	ofs		<< cc2_p << ends;
	bos		<< cc2_p << ends;					// write a const string const pointer(term by a '\0')

	cout	<< cc2_p << endl;

	// STL string
	//-----------

	string stl_string	= "STL string";

	ofs		<< stl_string << ends;
	bos		<< stl_string << ends;					// write a stl std::string

	cout	<< stl_string << endl;

	// char output
	//------------
	char c = 'A';

	ofs.put(c);
	bos.put(c);									// output the char

	cout << c << endl;

	cout << "ofstream tellp - " << ofs.tellp() << " bofstream tellp - " << bos.tellp() << endl;		// output the file position

	if (!bos || !ofs)
	{
		cout << "**********************************************************" << endl;
		cout << "******************Error writing to file*******************" << endl;
		cout << "**********************************************************" << endl;
	}
}

////////////////////////////////

void Input_Binary()
{
	int index;			// looping var

	cout << "Binary Input" << endl;
	cout << "------------" << endl;

	bifstream	is("new.txt", ios::binary);

	if (!is)
	{
		cout << "*********************************************************" << endl;
		cout << "**********************Error reading file*****************" << endl;
		cout << "*********************************************************" << endl;
	}

	// basic type input
	//------------------

	int		i;
	float	f;
	double	d;

	is >> i >> f >> d;							// read basic types like this

	cout << i << " " << f << " " << d << endl;

	// basic array input
	//-------------------

	float f_a[5];

	is.read(f_a, 5*sizeof(float));				// read an array like this (no cast needed)

	for (index = 0; index < 5; ++index)
		cout << f_a[index] << " ";
	cout << endl;

	// basic pointer input
	//--------------------
	float *f_p	= new float[5];

	for (index = 0; index < 5; ++index)			// This is the slower way of reading an array
		is >> f_p[index];

	for (index = 0; index < 5; ++index)
		cout << f_p[index] << " ";
	cout << endl;

	delete	[]f_p;
	f_p	= NULL;

	// string input - char buffer
	//---------------------------

	char buffer[20];

	//	is >> buffer;					// Never use >> to read char *!!!!!, should use get or getline
	is.getline(buffer, 20);				// use getline read in a string (term char is '\n')

	cout << buffer << endl;

	// string input - raw pointer
	//----------------------------

	char *c_p = new char[20];

	is.getline(c_p, 20);

	cout << c_p << endl;

	delete	[]c_p;
	c_p	= NULL;

	// string input - const string
	//----------------------------

	char const_buf[20];

	is.getline(const_buf, 20);			// read in the value outputted by a const string(term char is '\n')

	cout << const_buf << endl;

	// const string pointer
	//---------------------

	string const_s1;

	is.getline(const_s1);				// better way to store strings (term char is '\0') - no more worries about buffer overrun

	cout << const_s1.c_str() << endl;

	string stl_s2;

	is >> stl_s2;						// a even better way to read a string into a stl std::string

	cout << stl_s2.c_str() << endl;

	// STL string
	//------------

	string	stl_string;

	is >> stl_string;					// read in the output by a STL string

	cout << stl_string.c_str() << endl;

	// single char operations
	//--------------------------
	char c;

	is >> c;
	cout << "Getting a char " << c << endl;

	//	is.putback(c);					// put back the extracted character
	//	is >> c;						// a better way to extract a char
	//	cout << "Putting it back and retriving char " << c << endl;

	cout << "bifstream tellg - " << is.tellg() << endl;	// output the file position

	if (!is)
	{
		cout << "*************************************************************" << endl;
		cout << "*****************Error reading from file*********************" << endl;
		cout << "*************************************************************" << endl;
	}
}